<%@ WebHandler Language="C#" Class="ImageGrabber" %>

using System;
using System.Web;
using System.Data;
using System.Data.SqlClient;
using System.Web.Caching;
using System.Configuration;

public class ImageGrabber : IHttpHandler
{
    public void ProcessRequest (HttpContext context)
    {
        // Grab the comic ID from the query string and validate it
        string id = context.Request["ComicID"];
        if (String.IsNullOrEmpty (id))
            return;

        int ComicID = -1;
        try {
            ComicID = Convert.ToInt32 (id);
        }
        catch (FormatException) {
            return;
        }

        // Check for the image in the application cache
        string key = ComicID.ToString ();
        byte[] image = (byte[]) context.Cache[key];

        // If the image isn't in the cache, get it from the database
        if (image == null) {
	    SqlConnection connection = new SqlConnection (ConfigurationManager.ConnectionStrings["MyComicsConnectionString"].ConnectionString);
            try
            {
                connection.Open();
                SqlCommand command = new SqlCommand ("SELECT CoverImage FROM Comics WHERE ComicID=@ComicID", connection);
        		command.Parameters.Add ("@ComicID", SqlDbType.SmallInt).Value = ComicID;
                image = (byte[]) command.ExecuteScalar ();

                // Cache the image and include a SqlCacheDependency
                if (image != null) {
                    SqlCacheDependency dependency = new SqlCacheDependency ("MyComics", "Comics");
                    context.Cache.Insert (key, image, dependency, DateTime.Now.AddMinutes (60), Cache.NoSlidingExpiration);
                }
            }
            finally
            {
                connection.Close();
            }
        }

        // Return the image in the HTTP response
        if (image != null) {
            context.Response.ContentType = "image/jpeg";
            context.Response.OutputStream.Write (image, 0, image.Length);
        }
    }
 
    public bool IsReusable
    {
        get { return false; }
    }
}